<?php
/**
 * @author dinaminis.lt
 * @version 1.0.9
 * @copyright 2015 Dinaminis.lt
 */
if (!defined('_PS_VERSION_'))
  exit;


class VenipakCarrier extends CarrierModule {

    private $_postErrors = array();

    const ADDRESS_WAREHOUSE = 'warehouse';
    const ADDRESS_SENDER = 'sender';
    const MANIFEST_STATUS_OPEN = 1;
    const MANIFEST_STATUS_CLOSED = 0;

    private static $codmodule = 'cashondelivery';
    
    public  $id_carrier;

    public $addressFields = array('VENIPAK_ADDRESS_TITLE', 'VENIPAK_ADDRESS_NAME', 'VENIPAK_ADDRESS_COMPANY', 'VENIPAK_ADDRESS_ADDRESS', 'VENIPAK_ADDRESS_CITY', 'VENIPAK_ADDRESS_COUNTRY', 'VENIPAK_ADDRESS_POSTCODE', 'VENIPAK_ADDRESS_PHONE_NUMBER', 'VENIPAK_ADDRESS_PERSON', 'VENIPAK_ADDRESS_IS_DEFAULT');

    private static $_classMap = array(
        'VeniManifest' => 'classes/VeniManifest.php',
        'VeniOrder' => 'classes/VeniOrder.php',
        'VenipakAPI' => 'classes/VenipakAPI.php'
    );

    private $yesno = array();
    private $addressFormFields = array();

    public function __construct()
    {
        $this->name = 'venipakcarrier';
        $this->tab = 'shipping_logistics';
        $this->version = '1.0.9';
        $this->author = 'Dinaminis.lt';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.5', 'max' => _PS_VERSION_);
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('venipak');
        $this->description = $this->l('Delivery method using Venipak.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        if (self::isInstalled($this->name))
		{
			// Getting carrier list
			global $cookie;
			$carriers = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);

			// Saving id carrier list
			$id_carrier_list = array();
			foreach($carriers as $carrier)
				$id_carrier_list[] .= $carrier['id_carrier'];

			// Testing if Carrier Id exists
			$warning = array();
			if (!in_array((int)(Configuration::get('VENIPAK_CARRIER_ID')), $id_carrier_list))
				$warning[] .= $this->l('"Venipak carrier"').' ';
			if (!Configuration::get('VENIPAK_API_LOGIN'))
				$warning[] .= $this->l('"Venipak API login is required."').' ';
            if(!Configuration::get('VENIPAK_API_PASSWORD') || Configuration::get('VENIPAK_API_PASSWORD')==''){
                $warning[] .= $this->l('"Venipak API password"');
            }
			if (count($warning)){
				$this->warning .= implode(' , ',$warning).$this->l('must be configured to use this module correctly').' ';
            }
		}
    }

    public function install()
    {
        if (Shop::isFeatureActive())
            Shop::setContext(Shop::CONTEXT_ALL);

        $carrierConfig = array(
            0 => array('name' => 'Venipak carrier',
                'id_tax_rules_group' => 0,
                'active' => true,
                'deleted' => 0,
                'shipping_handling' => false,
                'range_behavior' => 0,
                'delay' => array('lt' => 'Venipak aprašymas', 'en' => 'Venipak description', Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) => 'Venipak description'),
                'id_zone' => 1,
                'is_module' => true,
                'shipping_external' => true,
                'external_module_name' => 'venipakcarrier',
                'need_range' => true
            )
        );

        // Install SQL
        include(dirname(__FILE__).'/sql-install.php');
        foreach ($sql as $s)
            if (!Db::getInstance()->Execute($s))
                return false;

        $venipakCarrierId = $this->installExternalCarrier($carrierConfig[0]);
        Configuration::updateValue('VENIPAK_CARRIER_ID', (int)$venipakCarrierId);
        if (!parent::install() ||
          !$this->registerHook('paymentConfirm') ||
          !$this->registerHook('extraCarrier') ||
          !$this->registerHook('displayHeader') ||
          !$this->registerHook('ActionAdminControllerSetMedia') ||
          !$this->registerHook('displayAdminOrder') ||
          !$this->registerHook('updateCarrier') ||
          !$this->registerHook('displayBeforePayment') ||
          !$this->installVenipakTab())
            return false;

        //save new order status
        $this->createOrderState();

        return true;
    }

    public static function installExternalCarrier($config)
	{
		$carrier = new Carrier();
		$carrier->name = $config['name'];
		$carrier->id_tax_rules_group = $config['id_tax_rules_group'];
		$carrier->id_zone = $config['id_zone'];
		$carrier->active = $config['active'];
		$carrier->deleted = $config['deleted'];
		$carrier->delay = $config['delay'];
		$carrier->shipping_handling = $config['shipping_handling'];
		$carrier->range_behavior = $config['range_behavior'];
		$carrier->is_module = $config['is_module'];
		$carrier->shipping_external = $config['shipping_external'];
		$carrier->external_module_name = $config['external_module_name'];
		$carrier->need_range = $config['need_range'];
        $carrier->url = 'http://go.venipak.com/ws/tracking.php?type=1&output=html&code=@';

		$languages = Language::getLanguages(true);
		foreach ($languages as $language)
		{
			if ($language['iso_code'] == 'lt')
				$carrier->delay[(int)$language['id_lang']] = $config['delay'][$language['iso_code']];
			if ($language['iso_code'] == 'en')
				$carrier->delay[(int)$language['id_lang']] = $config['delay'][$language['iso_code']];
			if ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')))
				$carrier->delay[(int)$language['id_lang']] = $config['delay'][$language['iso_code']];
		}

		if ($carrier->add())
		{
			$groups = Group::getGroups(true);
			foreach ($groups as $group)
				Db::getInstance()->autoExecute(_DB_PREFIX_.'carrier_group', array('id_carrier' => (int)($carrier->id), 'id_group' => (int)($group['id_group'])), 'INSERT');

			$rangePrice = new RangePrice();
			$rangePrice->id_carrier = $carrier->id;
			$rangePrice->delimiter1 = '0';
			$rangePrice->delimiter2 = '10000';
			$rangePrice->add();

			$rangeWeight = new RangeWeight();
			$rangeWeight->id_carrier = $carrier->id;
			$rangeWeight->delimiter1 = '0';
			$rangeWeight->delimiter2 = '10000';
			$rangeWeight->add();

			$zones = Zone::getZones(true);
			foreach ($zones as $zone)
			{
				Db::getInstance()->autoExecute(_DB_PREFIX_.'carrier_zone', array('id_carrier' => (int)($carrier->id), 'id_zone' => (int)($zone['id_zone'])), 'INSERT');
				Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_.'delivery', array('id_carrier' => (int)($carrier->id), 'id_range_price' => (int)($rangePrice->id), 'id_range_weight' => NULL, 'id_zone' => (int)($zone['id_zone']), 'price' => '0'), 'INSERT');
				Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_.'delivery', array('id_carrier' => (int)($carrier->id), 'id_range_price' => NULL, 'id_range_weight' => (int)($rangeWeight->id), 'id_zone' => (int)($zone['id_zone']), 'price' => '0'), 'INSERT');
			}

			// Copy Logo
			if (!copy(dirname(__FILE__).'/carrier.png', _PS_SHIP_IMG_DIR_.'/'.(int)$carrier->id.'.jpg'))
				return false;

			// Return ID Carrier
			return (int)($carrier->id);
		}

		return false;
	}

    public function uninstall()
    {
        // Uninstall
        if (!parent::uninstall() ||
          //@TODO delete configurations!
          !$this->unregisterHook('updateCarrier')||
          !$this->unregisterHook('displayHeader') ||
          !$this->unregisterHook('extraCarrier') ||
          !$this->unregisterHook('ActionAdminControllerSetMedia') ||
          !$this->unregisterHook('displayAdminOrder') ||
          !$this->uninstallVenipakTab())
            return false;

        // Delete External Carrier
        $venipakCarrier = new Carrier((int)(Configuration::get('VENIPAK_CARRIER_ID')));

        // If external carrier is default set other one as default
        if (Configuration::get('PS_CARRIER_DEFAULT') == (int)($venipakCarrier->id))
        {
            global $cookie;
            $carriersD = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);
            foreach($carriersD as $carrierD)
                if ($carrierD['active'] AND !$carrierD['deleted'] AND ($carrierD['name'] != $this->_config['name']))
                    Configuration::updateValue('PS_CARRIER_DEFAULT', $carrierD['id_carrier']);
        }

        // Then delete Carrier
        $venipakCarrier->deleted = 1;
        $venipakCarrier->active = 0;
        if (!$venipakCarrier->update())
            return false;

        return true;
    }

    public function createOrderState()
	{
		if (!Configuration::get('VENIPAK_ORDER_STATUS'))
		{
			$order_state = new OrderState();
			$order_state->name = array();

			foreach (Language::getLanguages() as $language)
			{
				if (Tools::strtolower($language['iso_code']) == 'lt')
					$order_state->name[$language['id_lang']] = 'Venipak - išsaugota';
				else
					$order_state->name[$language['id_lang']] = 'Venipak - saved';
			}

			$order_state->send_email = false;
			$order_state->color = '#eeab00';
			$order_state->hidden = false;
			$order_state->delivery = false;
			$order_state->logable = true;
			$order_state->invoice = false;

			$order_state->add();
			Configuration::updateValue('VENIPAK_ORDER_STATUS', (int)$order_state->id);
		}
	}

    private function installVenipakTab()
    {
        // erp tab name by lang
        $venipak_name_tab = array('en'=>'Venipak', 'lt'=>'Venipak', Language::getIsoById(Configuration::get('PS_LANG_DEFAULT'))=>'Venipak');
        $tab = new Tab();
        $tab->name = array();
        foreach (Language::getLanguages(false) as $language)
        {
            $iso_code = array_key_exists($language['iso_code'], $venipak_name_tab) ? $language['iso_code'] : 'en';
            $tab->name[$language['id_lang']] = $venipak_name_tab[$iso_code];
        }
        $tab->class_name = 'AdminVenipak';
        $tab->module = $this->name;
        $tab->id_parent = (int)Tab::getIdFromClassName('AdminParentShipping');
        if (!$tab->save())
        {
            $this->_errors[] = $this->l('Error while creating venipak tab.');
            return false;
        }
        return true;
    }

    private function uninstallVenipakTab()
    {
        $id_tab = Tab::getIdFromClassName('AdminVenipak');
        if ($id_tab != 0)
        {
            $tab = new Tab($id_tab);
            if (!$tab->delete())
            {
                $this->_errors[] = $this->l('Error while uninstalling venipak tab !');
                return false;
            }
        }
        return true;
    }

    public function getContent()
    {
        $this->yesno = array(
            array('id'=>0,'name'=>$this->l('No')),
            array('id'=>1,'name'=>$this->l('Yes'))
        );
        
        if(isset($_GET['VENIPAK_ADDRESS_ID']) || (isset($_GET['venipage']) && $_GET['venipage']=='veni_address') || isset($_GET['venipak_address_type'])){
            if(isset($_GET['deletevenisender'])){
                return $this->actiondeleteAddress();
            }else{
                return $this->renderAddressForm();
            }
        }elseif(isset($_GET['venipage']) && $_GET['venipage']=='saveorder'){
            //in POST will be order data

        }else{
            return $this->postProcess().$this->displayForm();
        }
    }

    public function postProcess(){
        $output = null;

        if(isset($_GET['message'])){
            $output .= $this->displayConfirmation(Tools::getValue('message'));
        }

        if (Tools::isSubmit('submit'.$this->name))
        {
            if (!$this->_postValidation()){
                $output .= $this->displayError($this->l('Invalid Configuration value'));
                $output .= $this->displayError(implode('<br />',$this->_postErrors));
            } else
            {

                Configuration::updateValue('VENIPAK_API_PASSWORD', strval(Tools::getValue('VENIPAK_API_PASSWORD')));
                Configuration::updateValue('VENIPAK_API_LOGIN', strval(Tools::getValue('VENIPAK_API_LOGIN')));
                Configuration::updateValue('VENIPAK_API_ID_CODE', strval(Tools::getValue('VENIPAK_API_ID_CODE')));
                Configuration::updateValue('VENIPAK_API_URL', strval(Tools::getValue('VENIPAK_API_URL')));
                $languages = Language::getLanguages(false);

                Configuration::updateValue('VENIPAK_SHOW_RETURN_DOCS', strval(Tools::getValue('VENIPAK_SHOW_RETURN_DOCS')));
                Configuration::updateValue('VENIPAK_SHOW_CHECK_ID', strval(Tools::getValue('VENIPAK_SHOW_CHECK_ID')));
                Configuration::updateValue('VENIPAK_SHOW_PICKUP', strval(Tools::getValue('VENIPAK_SHOW_PICKUP')));
                Configuration::updateValue('VENIPAK_SHOW_COD', strval(Tools::getValue('VENIPAK_SHOW_COD')));
                Configuration::updateValue('VENIPAK_ORDER_STATUS', strval(Tools::getValue('VENIPAK_ORDER_STATUS')));
                Configuration::updateValue('VENIPAK_LABELS_FORMAT', strval(Tools::getValue('VENIPAK_LABELS_FORMAT')));
                Configuration::updateValue('VENIPAK_SHOW_DELIVERY_TYPES', strval(Tools::getValue('VENIPAK_SHOW_DELIVERY_TYPES')));
                Configuration::updateValue('VENIPAK_DELIVERY_TYPES', implode(':',Tools::getValue('VENIPAK_DELIVERY_TYPES')));

                Configuration::updateValue('VENIPAK_SHOW_COMMENT_DOOR_CODE', pSQL(Tools::getValue('VENIPAK_SHOW_COMMENT_DOOR_CODE')));
                Configuration::updateValue('VENIPAK_SHOW_COMMENT_OFFICE_NO', pSQL(Tools::getValue('VENIPAK_SHOW_COMMENT_OFFICE_NO')));
                Configuration::updateValue('VENIPAK_SHOW_COMMENT_WAREHOUSE_NO', pSQL(Tools::getValue('VENIPAK_SHOW_COMMENT_WAREHOUSE_NO')));
                Configuration::updateValue('VENIPAK_SHOW_COMMENT_CALL', pSQL(Tools::getValue('VENIPAK_SHOW_COMMENT_CALL')));

                $output .= $this->displayConfirmation($this->l('Settings updated'));
                //process update of last pack_id by API number, if needed
                if(Tools::getValue('VENIPAK_LAST_PACK_NO_CHANGE')=='1' && Tools::getValue('VENIPAK_LAST_PACK_NO_NUMBER')!=''){
                    $output .= $this->updateMaxPackNo(Tools::getValue('VENIPAK_LAST_PACK_NO_NUMBER'), Configuration::get('VENIPAK_API_ID_CODE'));
                }
            }
        }
        return $output;
    }

    private function _postValidation(){

        $apiPassword = strval(Tools::getValue('VENIPAK_API_PASSWORD'));
        $apiLogin = strval(Tools::getValue('VENIPAK_API_LOGIN'));
        $apiIdCode = strval(Tools::getValue('VENIPAK_API_ID_CODE'));
        $usedPackValue = strval(Tools::getValue('VENIPAK_USED_PACK_SIGN'));

        if($apiPassword==NULL){
            $this->_postErrors[]=$this->l("API Password is required.");
        }
        if($apiLogin==NULL){
            $this->_postErrors[]=$this->l("API login is required.");
        }
        if($apiIdCode==NULL){
            $this->_postErrors[]=$this->l("API ID code is required.");
        }
        return empty($this->_postErrors);
    }

    public function displayForm(){

        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        $orderStatusesOriginal = OrderStateCore::getOrderStates(Context::getContext()->language->id);
        $orderStatuses = array(array('id'=>'','name'=>'---'));
        foreach($orderStatusesOriginal as $status){
            $orderStatuses[]=array('id'=>$status['id_order_state'], 'name'=>$status['name']);
        }
        $deliveryTypes = array();
        foreach($this->getDeliveryTimes() as $key=>$value){
            $deliveryTypes[]=array('id'=>$key, 'name'=>$value);
        }

        // Init Fields form array
        $fields_form[0] = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Settings'),
					'icon' => 'icon-cogs'
				),
				'input' => array(
                    array(
						'type' => 'text',
						'label' => $this->l('API URL'),
						'name' => 'VENIPAK_API_URL',
					),
					array(
						'type' => 'text',
						'label' => $this->l('API Login'),
						'name' => 'VENIPAK_API_LOGIN'
					),
                    array(
						'type' => 'text',
						'label' => $this->l('API Password'),
						'name' => 'VENIPAK_API_PASSWORD'
					),
                    array(
						'type' => 'text',
						'label' => $this->l('API ID'),
						'name' => 'VENIPAK_API_ID_CODE'
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Set last used PACK ID'),
						'name' => 'VENIPAK_LAST_PACK_NO_CHANGE',
                        'options' => array(
                            'query' => $this->yesno,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Pack No. (last 7 digits):'),
						'name' => 'VENIPAK_LAST_PACK_NO_NUMBER',
                        'form_group_class' => 'hide',
                        'options'=>array(
                        )
					),
                    
				)
			),
		);

        $fields_form[] = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Additional settings'),
					'icon' => 'icon-cogs'
				),
				'input' => array(
                    array(
						'type' => 'select',
						'label' => $this->l('Show "return accompanying documents"'),
						'name' => 'VENIPAK_SHOW_RETURN_DOCS',
                        'options' => array(
                            'query' => $this->yesno,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Show "Check ID"'),
						'name' => 'VENIPAK_SHOW_CHECK_ID',
                        'options' => array(
                            'query' => $this->yesno,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Show "Select sender location"'),
						'name' => 'VENIPAK_SHOW_PICKUP',
                        'options' => array(
                            'query' => $this->yesno,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Enable C.O.D.'),
						'name' => 'VENIPAK_SHOW_COD',
                        'options' => array(
                            'query' => $this->yesno,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Order status after saving'),
						'name' => 'VENIPAK_ORDER_STATUS',
                        'options' => array(
                            'query' => $orderStatuses,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Format of labels'),
						'name' => 'VENIPAK_LABELS_FORMAT',
                        'options' => array(
                            'query' => array(array('id'=>'100x150', 'name'=>'100x150'), array('id'=>'a4', 'name'=>'a4')),
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Show available delivery types?'),
						'name' => 'VENIPAK_SHOW_DELIVERY_TYPES',
                        'options' => array(
                            'query' => $this->yesno,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Show door code input?'),
						'name' => 'VENIPAK_SHOW_COMMENT_DOOR_CODE',
                        'options' => array('query' => $this->yesno,'id' => 'id','name' => 'name'),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Show office number input?'),
						'name' => 'VENIPAK_SHOW_COMMENT_OFFICE_NO',
                        'options' => array('query' => $this->yesno,'id' => 'id','name' => 'name'),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Show warehouse number input?'),
						'name' => 'VENIPAK_SHOW_COMMENT_WAREHOUSE_NO',
                        'options' => array('query' => $this->yesno,'id' => 'id','name' => 'name'),
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Show call before delivery input?'),
						'name' => 'VENIPAK_SHOW_COMMENT_CALL',
                        'options' => array('query' => $this->yesno,'id' => 'id','name' => 'name'),
					),
                    array(
						'type' => 'select',
                        'multiple' => true,
						'label' => $this->l('Available delivery types'),
						'name' => 'VENIPAK_DELIVERY_TYPES[]',
                        'options' => array(
                            'query' => $deliveryTypes,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    ),
                'submit' => array(
                    'title' => $this->l('Save'),                       // This is the button that saves the whole fieldset.
                    'class' => 'button'
                )
                ),

            );


        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $languages = Language::getLanguages(false);
        foreach ($languages as $k => $language){
            $languages[$k]['is_default'] = (int)($language['id_lang'] == Configuration::get('PS_LANG_DEFAULT'));
        }
        $helper->languages = $languages;
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' =>
            array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        $helper->fields_value=$this->getConfigFieldsValues();

        return $helper->generateForm($fields_form).$this->warehousesList().$this->sendersList();
    }

    public function renderAddressForm(){
        $this->addressFormFields = array(
                    array(
						'type' => 'text',
						'label' => $this->l('Address title'),
						'name' => 'VENIPAK_ADDRESS_TITLE',
					),
					array(
						'type' => 'text',
						'label' => $this->l('Company name'),
						'name' => 'VENIPAK_ADDRESS_NAME',
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Company code'),
						'name' => 'VENIPAK_ADDRESS_COMPANY',
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Address'),
						'name' => 'VENIPAK_ADDRESS_ADDRESS',
					),
                    array(
						'type' => 'text',
						'label' => $this->l('City'),
						'name' => 'VENIPAK_ADDRESS_CITY',
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Country'),
						'name' => 'VENIPAK_ADDRESS_COUNTRY',
                        'options' => array(
                            'query' => $this->getSupportedCountries(),
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Postcode'),
						'name' => 'VENIPAK_ADDRESS_POSTCODE',
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Phone number'),
						'name' => 'VENIPAK_ADDRESS_PHONE_NUMBER',
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Contact person'),
						'name' => 'VENIPAK_ADDRESS_PERSON',
					),
                    array(
						'type' => 'select',
						'label' => $this->l('Make default'),
						'name' => 'VENIPAK_ADDRESS_IS_DEFAULT',
                        'options' => array(
                            'query' => $this->yesno,
                            'id' => 'id',
                            'name' => 'name'
                        ),
					),
                    array(
						'type' => 'hidden',
						'name' => 'VENIPAK_ADDRESS_TYPE',
					),
                );
        $output = '<div class="panel"><span style="float: left;"><a href="'.AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules').'"><i class="process-icon-back"></i><div>'.$this->l('Back').'</div></a></span><div class="clear clearfix"></div></div>';
        if (Tools::isSubmit('venipakaddresssave'))
        {
            if($this->validateAddress()){

                if($this->saveAddress()){
                    //$output .= $this->displayConfirmation($this->l('Warehouse saved.'));
                    $message = $this->l('Warehouse saved.');
                    if(strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_TYPE'))))=='sender'){
                        $message = $this->l('Sender saved.');
                    }
                    Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules').'&message='.$message);
                }
            }else{
                $output .= $this->displayError($this->l('Please recheck warehouse data:').'<br />'.implode(' <br /> ',$this->_postErrors));
            }
            //return $output;
        }

        $veniAddressType = 'warehouse';
        $warehouse = false;

        if(isset($_GET['VENIPAK_ADDRESS_ID'])){
            //it's update
            //get warehouse info
            $warehouse = $this->getAddress(Tools::getValue('VENIPAK_ADDRESS_ID'));
            if($warehouse==false){
                return $this->displayError("No such warehouse is found.");
            }
            $veniAddressType = $warehouse['type'];
        }else{
            //it's new
            if(isset($_GET['venipak_address_type']) && ($_GET['venipak_address_type']==self::ADDRESS_SENDER || $_GET['venipak_address_type']==self::ADDRESS_WAREHOUSE )){
                $veniAddressType = $_GET['venipak_address_type'];
            }else{
                return $this->displayError("Wrong type of address.");
            }
        }

        $helper = new HelperForm();
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name.'&venipage=veni_address&venipak_address_type='.$veniAddressType;
        if($warehouse!=false){
            $helper->currentIndex .= '&VENIPAK_ADDRESS_ID='.$warehouse['id'];
        }

        // Language
        $languages = Language::getLanguages(false);
        foreach ($languages as $k => $language){
            $languages[$k]['is_default'] = (int)($language['id_lang'] == Configuration::get('PS_LANG_DEFAULT'));
        }
        $helper->languages = $languages;
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;


        $fields_form[0] = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Address form'),
					'icon' => 'icon-cogs'
				),
				'input' => $this->addressFormFields,
                'submit' => array(
                    'title' => ($warehouse!=false ? $this->l('Update') : $this->l('Save')),                       // This is the button that saves the whole fieldset.
                    'class' => 'button',
                    'name'=>'venipakaddresssave'
                )
            )
        );

        //set null all warehouse fields
        foreach($this->addressFields as $f){
            $helper->fields_value[$f]=false;
        }
        $helper->fields_value['VENIPAK_ADDRESS_TYPE'] = $veniAddressType;

            //get warehouse info
            if(!empty($warehouse)){
                //add ID field
                $fields_form[0]['form']['input'][]=array(
                    'type'=>'hidden',
                    'name'=>'VENIPAK_ADDRESS_ID'
                );
                $helper->fields_value['VENIPAK_ADDRESS_TITLE'] = $warehouse['address_title'];
                $helper->fields_value['VENIPAK_ADDRESS_NAME'] = $warehouse['name'];
                $helper->fields_value['VENIPAK_ADDRESS_COMPANY'] = $warehouse['company'];
                $helper->fields_value['VENIPAK_ADDRESS_ADDRESS'] = $warehouse['address'];
                $helper->fields_value['VENIPAK_ADDRESS_CITY'] = $warehouse['city'];
                $helper->fields_value['VENIPAK_ADDRESS_COUNTRY'] = $warehouse['country'];
                $helper->fields_value['VENIPAK_ADDRESS_POSTCODE'] = $warehouse['postcode'];
                $helper->fields_value['VENIPAK_ADDRESS_PHONE_NUMBER'] = $warehouse['phone_number'];
                $helper->fields_value['VENIPAK_ADDRESS_PERSON'] = $warehouse['person'];
                $helper->fields_value['VENIPAK_ADDRESS_IS_DEFAULT'] = $warehouse['is_default'];
                $helper->fields_value['VENIPAK_ADDRESS_ID'] = $warehouse['id'];
            }else{
                $helper->fields_value['VENIPAK_ADDRESS_TITLE'] = Tools::getValue('VENIPAK_ADDRESS_TITLE');
                $helper->fields_value['VENIPAK_ADDRESS_NAME'] = Tools::getValue('VENIPAK_ADDRESS_NAME');
                $helper->fields_value['VENIPAK_ADDRESS_COMPANY'] = Tools::getValue('VENIPAK_ADDRESS_COMPANY');
                $helper->fields_value['VENIPAK_ADDRESS_ADDRESS'] = Tools::getValue('VENIPAK_ADDRESS_ADDRESS');
                $helper->fields_value['VENIPAK_ADDRESS_CITY'] = Tools::getValue('VENIPAK_ADDRESS_CITY');
                $helper->fields_value['VENIPAK_ADDRESS_COUNTRY'] = Tools::getValue('VENIPAK_ADDRESS_COUNTRY');
                $helper->fields_value['VENIPAK_ADDRESS_POSTCODE'] = Tools::getValue('VENIPAK_ADDRESS_POSTCODE');
                $helper->fields_value['VENIPAK_ADDRESS_PHONE_NUMBER'] = Tools::getValue('VENIPAK_ADDRESS_PHONE_NUMBER');
                $helper->fields_value['VENIPAK_ADDRESS_PERSON'] = Tools::getValue('VENIPAK_ADDRESS_PERSON');
                $helper->fields_value['VENIPAK_ADDRESS_IS_DEFAULT'] = Tools::getValue('VENIPAK_ADDRESS_IS_DEFAULT');
            }



        return $output.$helper->generateForm($fields_form);
    }

    public function actiondeleteAddress(){
        if((int)Tools::getValue('VENIPAK_ADDRESS_ID')>0){
            $addressId = (int)Tools::getValue('VENIPAK_ADDRESS_ID');
            //check if exists
            if($this->getAddress($addressId)){
                $this->deleteAddress($addressId);
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'));
            }
        }
    }


    public function warehousesList()
    {
        $content = array();
        //collect all current warehouses
        $warehouses = $this->getWarehouses();
        if(!empty($warehouses)){
            foreach($warehouses as $wh){
                $content[] = array(
                    'VENIPAK_ADDRESS_ID'=>$wh['id'],
                    'address_title' => $wh['address_title'],
                    'name'=>$wh['name'],
                    'address'=>$wh['address'],
                    'is_default'=> ($wh['is_default']=='1' ? $this->l('Yes') : $this->l('No'))
                );
            }
        }

        $fields_list = $this->getListColumns();

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->show_toolbar = true;
        $helper->module = $this;
        $helper->listTotal = count($content);
        $helper->identifier = 'VENIPAK_ADDRESS_ID';
        $helper->title = 'Warehouses ';
        $helper->actions = array('edit','delete');
        $helper->table = 'venisender';
        //$helper->table = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper->toolbar_btn['new'] = array(
				'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules').'&configure='.$this->name.'&venipage=veni_address&venipak_address_type='.self::ADDRESS_WAREHOUSE,
				'desc' => $this->l('New')
			);

        return $helper->generateList($content, $fields_list);
    }

    public function sendersList()
    {
        $content = array();
        //collect all current warehouses
        $warehouses = $this->getSenders();
        if(!empty($warehouses)){
            foreach($warehouses as $wh){
                $content[] = array(
                    'VENIPAK_ADDRESS_ID'=>$wh['id'],
                    'address_title' => $wh['address_title'],
                    'name'=>$wh['name'],
                    'address'=>$wh['address'],
                    'is_default'=> ($wh['is_default']=='1' ? $this->l('Yes') : $this->l('No'))
                );
            }
        }

        $fields_list = $this->getListColumns();

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->show_toolbar = true;
        $helper->module = $this;
        $helper->listTotal = count($content);
        $helper->identifier = 'VENIPAK_ADDRESS_ID';
        $helper->title = 'Senders ';
        $helper->actions = array('edit','delete');
        $helper->table = 'venisender';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper->toolbar_btn['new'] = array(
				'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules').'&configure='.$this->name.'&venipage=veni_address&venipak_address_type='.self::ADDRESS_SENDER,
				'desc' => $this->l('New')
			);

        return $helper->generateList($content, $fields_list);
    }


    public function validateAddress(){
        foreach($this->addressFields as $f){
            if(strval(Tools::getValue($f))=='' && $f!='VENIPAK_ADDRESS_COMPANY'){
                //find nice field name
                $fname = '';
                foreach($this->addressFormFields as $af){
                    if($af['name']==$f){
                        $fname=$af['label']; break;
                    }
                }
                $this->_postErrors[]=$fname.' field is empty';
            }
        }
        $companyCode = Tools::getValue('VENIPAK_ADDRESS_COMPANY');
        if($companyCode!=''){
            if(!is_numeric($companyCode)){
                $this->_postErrors[]=$this->l('Company code must be numeric.');
            }
            if(strlen($companyCode)<7){
                $this->_postErrors[]=$this->l('Company code is too short (at least 7 digits).');
            }
        }
        
        if(Tools::getValue('VENIPAK_ADDRESS_COUNTRY')=='LV'){
            if(strlen(Tools::getValue('VENIPAK_ADDRESS_POSTCODE'))!=4){
                $this->_postErrors[]=$this->l('Invalid post code.');
            }
        }else{
            if(strlen(Tools::getValue('VENIPAK_ADDRESS_POSTCODE'))!=5){
                $this->_postErrors[]=$this->l('Invalid post code.');
            }
        }
        return empty($this->_postErrors);
    }

    public function saveAddress(){
        $addressData = array(
                'address_title'=>strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_TITLE')))),
                'name'=>strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_NAME')))),
                'company' => strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_COMPANY')))),
                'address' => strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_ADDRESS')))),
                'city' => strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_CITY')))),
                'country' => strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_COUNTRY')))),
                'postcode' => strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_POSTCODE')))),
                'phone_number' => strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_PHONE_NUMBER')))),
                'person' => strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_PERSON')))),
                'type' => strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_TYPE')))),
                'is_default' => 0,
            );
        if(Tools::getValue('VENIPAK_ADDRESS_ID')==false){
            //create
            $result = Db::getInstance()->insert('venipak_carrier_addresses', $addressData);
            $lastId = Db::getInstance()->Insert_ID();
            //check for default
            if(Tools::getValue('VENIPAK_ADDRESS_IS_DEFAULT')=='1'){
                Db::getInstance()->update('venipak_carrier_addresses', array('is_default'=>'0'), 'is_default=1 AND type="'.strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_TYPE')))).'"');
                Db::getInstance()->update('venipak_carrier_addresses', array('is_default'=>'1'), 'id='.$lastId);
            }
            return $result;
        }else{
            //update
            $upData = $addressData;
            unset($upData['type']);
            $result = Db::getInstance()->update('venipak_carrier_addresses', $upData,
            'id='.(int)Tools::getValue('VENIPAK_ADDRESS_ID'));
            if(Tools::getValue('VENIPAK_ADDRESS_IS_DEFAULT')=='1'){
                Db::getInstance()->update('venipak_carrier_addresses', array('is_default'=>'0'), 'is_default=1 AND type="'.strval(strip_tags(trim(Tools::getValue('VENIPAK_ADDRESS_TYPE')))).'"');
                Db::getInstance()->update('venipak_carrier_addresses', array('is_default'=>'1'), 'id='.(int)Tools::getValue('VENIPAK_ADDRESS_ID'));
            }
            return $result;
        }
    }



    public function getWarehouses(){
        $db = Db::getInstance();
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'venipak_carrier_addresses WHERE type="'.self::ADDRESS_WAREHOUSE.'" ORDER BY is_default DESC, id';
        return $db->executeS($sql);
    }
    public function getWarehouse($warehouseId){
        $db = Db::getInstance();
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'venipak_carrier_addresses WHERE type="'.self::ADDRESS_WAREHOUSE.'" AND id='.(int)$warehouseId.' ORDER BY is_default DESC, id';
        return $db->getRow($sql);
    }
    public function getDefaultWarehouse(){
        $db = Db::getInstance();
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'venipak_carrier_addresses WHERE type="'.self::ADDRESS_WAREHOUSE.'" AND is_default=1';
        return $db->getRow($sql);
    }
    public function getSenders(){
        $db = Db::getInstance();
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'venipak_carrier_addresses WHERE type="'.self::ADDRESS_SENDER.'" ORDER BY is_default DESC, id';
        return $db->executeS($sql);
    }
    public function getAddress($id){
        $db = Db::getInstance();
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'venipak_carrier_addresses WHERE id='.(int)$id;
        return $db->getRow($sql);
    }
    public function deleteAddress($id){
        $db = Db::getInstance();
        $sql = 'DELETE FROM ' . _DB_PREFIX_ . 'venipak_carrier_addresses WHERE id='.(int)$id;
        return $db->execute($sql);
    }
    /**
     * Inserts dummy pack in DB, from which new ones will be calculated
     * @param integer $maxPackNo
     * @param integer $apiId
     * @return bool
     */
    private function updateMaxPackNo($maxPackNo, $apiId){
        $maxPackNo = (int)$maxPackNo; //cast
        $apiId = (int)$apiId; //cast
        if($maxPackNo<0){
            return false;
        }
        //first check for dublicate
        $result = Db::getInstance()->getRow('SELECT * FROM ' . _DB_PREFIX_ . 'venipak_order_pack WHERE pack_no='.(int)$maxPackNo.' AND api_id='.(int)$apiId.'');
        if(!empty($result)){
            //such entry already exists
            return $this->displayError($this->l('Entry with such API ID and pack No. already exists.'));
        }
        $result = Db::getInstance()->insert('venipak_order_pack', array(
                    'pack_no' => $maxPackNo,
                    'api_id'=>$apiId,
                    'order_id' => 0,
                    'weight' => 0,
                    'sent' => 1
                ));
        return true;
    }

    /**
     * Returns prepared array of supported countries for dropdowns
     * @return array Array of supported countries
     */
    public function getSupportedCountries(){
        $supported_countries = array(
            array('id' => 'LT', 'name'=>'LT'),
            array('id' => 'EE', 'name'=>'EE'),
            array('id' => 'LV', 'name'=>'LV'),
            array('id' => 'PL', 'name'=>'PL'),
            );
        return $supported_countries;
    }

    private function getListColumns(){
        $fields_list = array(
            'VENIPAK_ADDRESS_ID' => array(
                'title' => 'ID',
                'align' => 'center',
                'class' => 'fixed-width-xs',
            ),
            'address_title' => array(
                'title' => $this->l('Address title'),
                'align' => 'center',
                'class' => 'fixed-width-md',
            ),
            'name' => array(
                'title' => $this->l('Name'),
                'align' => 'center',
                'class' => 'fixed-width-md',
            ),
            'address' => array(
                'title' => $this->l('Address'),
                'align' => 'center',
                'class' => 'fixed-width-md',
            ),
            'is_default' => array(
                'title' => $this->l('Is default?'),
                'align' => 'center',
                'class' => 'fixed-width-md',
            ),
        );
        return $fields_list;
    }

    /**
     *
     * @param int $warehouseId
     * @param Y-m-d $date
     */
    public function getCarrierCallByDate($warehouseId, $date){
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'venipak_carrier_call WHERE date(call_date)="'.pSQL(strval($date)).'" AND warehouse_id='.(int)$warehouseId;
        return Db::getInstance()->getRow($sql);
    }

    public function getCarrierCallFromDate($warehouseId, $date){
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'venipak_carrier_call WHERE date(call_date)>="'.pSQL(strval($date)).'" AND warehouse_id='.(int)$warehouseId;
        return Db::getInstance()->executeS($sql);
    }

    public function getCartComments($cartId){
        $db = Db::getInstance();
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'venipak_tmp_cart_comments WHERE cart_id="'.$cartId.'"';
        return $db->getRow($sql);
    }


    /*
	** Hook update carrier
	**
	*/
	public function hookupdateCarrier($params)
	{
		if ((int)($params['id_carrier']) == (int)(Configuration::get('VENIPAK_CARRIER_ID'))){
			Configuration::updateValue('VENIPAK_CARRIER_ID', (int)($params['carrier']->id));
        }
	}

    public function hookActionAdminControllerSetMedia($params){
        $this->context->controller->addJS($this->_path.'/views/js/venipak.js');
        $this->context->controller->addCSS($this->_path.'/views/css/venipak.css');
    }

    public function hookextraCarrier($params)
	{
        if(!empty($this->warning)){
            return false;
        }
            
        $id_cart = $params['cart']->id;
		
		$postal_code = $params['address']->postcode;
        global $smarty;

        if (isset($this->context) && method_exists($this->context->controller, 'addJS'))
		{
			$this->context->controller->addCss($this->_path.'views/front/venipak_public.css');
		}

        $availableDeliveryTypes = array();
        if(Configuration::get('VENIPAK_DELIVERY_TYPES')!=''){
            //try to explode
            $availableDeliveryTypes = explode(':',Configuration::get('VENIPAK_DELIVERY_TYPES'));
        }

        $smarty->assign(
            array(

                'id_cart' => $id_cart,
                'version' => _PS_VERSION_,
                'veni_carrier_id' => Configuration::get('VENIPAK_CARRIER_ID'),
                'id_address_delivery' => $params['cart']->id_address_delivery,
                'controllerurl' => $this->context->link->getModuleLink('venipakcarrier', 'venipakfrontcont'),
                'availableDeliveryTypes' => $availableDeliveryTypes,
                'allDeliveryTypes' => $this->getDeliveryTimes()
            )
        );

        return $this->display(__FILE__, 'tpl/front/carrier.tpl');
        
		
	}

    /**
     * Used to disable default COD module if selected carrier is Venipak carrier and COD in venipak settings is disabled
     */
    public function hookdisplayBeforePayment(){
        if(self::isInstalled($this->name) && !Module::isEnabled($this->name)){
            return;
        }
        $cart = Context::getContext()->cart;
        $carrierId = $cart->id_carrier;
        //default is "cashondelivery"
        $codModule = Module::getInstanceByName(self::$codmodule);

        if($carrierId==$this->id_carrier && Configuration::get('VENIPAK_SHOW_COD')=='0'){
            //disable "cod" module
            //payment modules are saved as instances, so just by setting property "active" to false is enough
            $codModule->active = false;
        }
    }


    public function getConfigFieldsValues()
	{
		$languages = Language::getLanguages(false);
		$fields = array();

        $fields['VENIPAK_API_LOGIN'] = Tools::getValue('VENIPAK_API_LOGIN', Configuration::get('VENIPAK_API_LOGIN'));
        $fields['VENIPAK_API_PASSWORD'] = Tools::getValue('VENIPAK_API_PASSWORD', Configuration::get('VENIPAK_API_PASSWORD'));
        $fields['VENIPAK_API_ID_CODE'] = Tools::getValue('VENIPAK_API_ID_CODE', Configuration::get('VENIPAK_API_ID_CODE'));
        $fields['VENIPAK_API_URL'] = Tools::getValue('VENIPAK_API_URL', Configuration::get('VENIPAK_API_URL'));
        $fields['VENIPAK_LAST_PACK_NO_CHANGE'] = Tools::getValue('VENIPAK_LAST_PACK_NO_CHANGE', 0);
        $fields['VENIPAK_LAST_PACK_NO_NUMBER'] = Tools::getValue('VENIPAK_LAST_PACK_NO_NUMBER', '');

        $fields['VENIPAK_SHOW_RETURN_DOCS'] = Tools::getValue('VENIPAK_SHOW_RETURN_DOCS', Configuration::get('VENIPAK_SHOW_RETURN_DOCS'));
        $fields['VENIPAK_SHOW_CHECK_ID'] = Tools::getValue('VENIPAK_SHOW_CHECK_ID', Configuration::get('VENIPAK_SHOW_CHECK_ID'));
        $fields['VENIPAK_SHOW_PICKUP'] = Tools::getValue('VENIPAK_SHOW_PICKUP', Configuration::get('VENIPAK_SHOW_PICKUP'));
        $fields['VENIPAK_SHOW_COD'] = Tools::getValue('VENIPAK_SHOW_COD', Configuration::get('VENIPAK_SHOW_COD'));
        $fields['VENIPAK_ORDER_STATUS'] = Tools::getValue('VENIPAK_ORDER_STATUS', Configuration::get('VENIPAK_ORDER_STATUS'));
        $fields['VENIPAK_LABELS_FORMAT'] = Tools::getValue('VENIPAK_LABELS_FORMAT', Configuration::get('VENIPAK_LABELS_FORMAT'));
        $fields['VENIPAK_DELIVERY_TYPES[]'] = Tools::getValue('VENIPAK_DELIVERY_TYPES', explode(':', Configuration::get('VENIPAK_DELIVERY_TYPES')));
        $fields['VENIPAK_SHOW_DELIVERY_TYPES'] = Tools::getValue('VENIPAK_SHOW_DELIVERY_TYPES', Configuration::get('VENIPAK_SHOW_DELIVERY_TYPES'));
        $fields['VENIPAK_SHOW_COMMENT_DOOR_CODE'] = Tools::getValue('VENIPAK_SHOW_COMMENT_DOOR_CODE', Configuration::get('VENIPAK_SHOW_COMMENT_DOOR_CODE'));
        $fields['VENIPAK_SHOW_COMMENT_OFFICE_NO'] = Tools::getValue('VENIPAK_SHOW_COMMENT_OFFICE_NO', Configuration::get('VENIPAK_SHOW_COMMENT_OFFICE_NO'));
        $fields['VENIPAK_SHOW_COMMENT_WAREHOUSE_NO'] = Tools::getValue('VENIPAK_SHOW_COMMENT_WAREHOUSE_NO', Configuration::get('VENIPAK_SHOW_COMMENT_WAREHOUSE_NO'));
        $fields['VENIPAK_SHOW_COMMENT_CALL'] = Tools::getValue('VENIPAK_SHOW_COMMENT_CALL', Configuration::get('VENIPAK_SHOW_COMMENT_CALL'));


		return $fields;
	}


    public function getOrderShippingCost($params, $shipping_cost)
    { 
      if ($this->id_carrier == (int)Configuration::get('VENIPAK_CARRIER_ID') && Module::isEnabled($this->name)){
        return (float)$shipping_cost;
      }
      return false; // carrier is not known
    }

    public function getOrderShippingCostExternal($params)
    {
        return $this->getOrderShippingCost($params, 0);
    }

    public function hookDisplayAdminOrder($id_order){
        //get weight of all products
        $order = new Order((int)$id_order['id_order']);
        $veniCarrierObj = new VenipakCarrier;
        if(Configuration::get('VENIPAK_CARRIER_ID')!=$order->id_carrier){ return false; }
        $manifestDates = array();
        $datetime = new DateTime(date('Y-m-d'));
        for($i=0;$i<6;$i++){

            $manifestDates[]=$datetime->format('Y-m-d');
            $datetime->modify('+1 day');
        }

        self::checkForClass('VeniOrder');
        self::checkForClass('VeniManifest');
        $veniOrderObj = new VeniOrder();
        $checkVeniOrder = $veniOrderObj->getOrderVeniData((int)$id_order['id_order']);
        $selectedComments = $this->getCartComments((int)$order->id_cart);

        $is_cod = ($order->module==VenipakCarrier::$codmodule);
        if(!empty($checkVeniOrder)){
            $checkVeniOrder['manifest_date'] = VeniManifest::getManifest($checkVeniOrder['manifest_no']);
        }
        $this->smarty->assign(array(
            'veni_warehouses'=>$this->getWarehouses(),
            'veni_senders'=>$this->getSenders(),
            'veni_manifest_dates'=>$manifestDates,
            'delivery_times'=>$this->getDeliveryTimes(),
            'total_weight'=>$order->getTotalWeight(),
            'total_paid_tax_incl'=>$order->total_paid_tax_incl,
            'venialldisabled'=>!empty($checkVeniOrder),
            'veniOrderInfo' => $checkVeniOrder,
            'selectedComments' => $selectedComments,
            'is_cod' => $is_cod,
            'order_id'=>(int)$id_order['id_order'],
            'venimoduleurl'=>$this->context->link->getModuleLink('venipakcarrier', 'venipakcont',array('action'=>'saveveniorder')),
            'veniprintlabelsurl'=>$this->context->link->getModuleLink('venipakcarrier', 'venipakcont',array('action'=>'printlabels')),
            'VENIPAK_SHOW_RETURN_DOCS'=>Configuration::get('VENIPAK_SHOW_RETURN_DOCS'),
            'VENIPAK_SHOW_CHECK_ID'=>Configuration::get('VENIPAK_SHOW_CHECK_ID'),
            'VENIPAK_SHOW_PICKUP'=>Configuration::get('VENIPAK_SHOW_PICKUP'),
            'VENIPAK_SHOW_COD'=>Configuration::get('VENIPAK_SHOW_COD'),
            'VENIPAK_DELIVERY_TYPES' => explode(':',Configuration::get('VENIPAK_DELIVERY_TYPES')),
        ));

        $veniForm = $this->display(__FILE__, 'tpl/blockinorder.tpl');
        return $veniForm;
    }


    public function getDeliveryTimes(){
        $deliveryTimes = array(
            'nwd' => $this->l('Standard'),
            'tswd' => $this->l('Same work day'),
            'nwd10' => $this->l('Next working day till 10:00'),
            'nwd12' => $this->l('Next working day till 12:00'),
            'nwd8_14' => $this->l('Next working day 8:00-14:00'),
            'nwd14_17' => $this->l('Next working day 14:00-17:00'),
            'nwd18_20' => $this->l('Next working day 18:00-22:00'),
            'nwd18a' => $this->l('Next working day after 18:00'),
            'sat' => $this->l('On saturday')
            );

        return $deliveryTimes;
    }


    public static function orderModuleIsCOD($orderModule){
        return (self::$codmodule==$orderModule);
    }


    /**
     *
     * @param string $date Format: Y-m-d
     * @return type
     */
    public function getTotalWeightByDate($date){
        $sql = 'SELECT SUM(product_weight * product_quantity) FROM ps_order_detail psorderdetail INNER JOIN ps_venipak_order_info AS veniorder ON psorderdetail.id_order=veniorder.order_id INNER JOIN ps_venipak_manifest as manifest ON veniorder.manifest_no=manifest.manifest_no WHERE manifest.manifest_date="'.pSQL($date).'" GROUP BY psorderdetail.id_order';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
		return (float)($result);
    }

    public function saveCarrierCall($warehouseId, $callDate, $callData){
        $result = Db::getInstance()->insert('venipak_carrier_call', array(
            'warehouse_id' => (int)$warehouseId,
            'call_date' => $callDate,
            'call_data' => $callData
        ));
        return $result;
    }

    

    public static function checkForClass($className){
        if(!class_exists($className)){
            if(isset(self::$_classMap[$className])){
                require_once dirname(__FILE__).'/'.self::$_classMap[$className];
            }
        }
    }

}